import httpx
import asyncio
import click
from typing import List

from argos import logging
from argos.logging import logger
from argos.checks import CheckNotFound, get_check_by_name

from argos.schemas import Task


async def complete_task(client: httpx.AsyncClient, task: dict) -> dict:
    task = Task(**task)
    check_class = get_check_by_name(task.check)
    check = check_class(client, task)
    result = await check.run()
    logger.error(f"{result=}")
    return {"id": task.id, "result": "completed"}


async def post_results(client: httpx.AsyncClient, server: str, results: List[dict]):
    response = await client.post(f"{server}/results", json={"results": results})

    if response.status_code == httpx.codes.OK:
        logger.info("Successfully posted results")
    else:
        logger.error(f"Failed to post results: {response.read()}")


async def run(server: str, max_tasks: int):
    tasks = []

    async with httpx.AsyncClient() as client:
        # Fetch the list of tasks
        response = await client.get(f"{server}/tasks")

        if response.status_code == httpx.codes.OK:
            # XXX Maybe we want to group the tests by URL ? (to issue one request per URL)
            data = response.json()
            logger.info(f"Received {len(data)} tasks from the server")

            for task in data:
                tasks.append(complete_task(client, task))

            # Run up to max_tasks concurrent tasks
            results = await asyncio.gather(*tasks)

            # Post the results
            await post_results(client, server, results)
        else:
            logger.error(f"Failed to fetch tasks: {response.read()}")


@click.command()
@click.option("--server", required=True, help="Server URL")
@click.option("--max-tasks", default=10, help="Maximum number of concurrent tasks")
@click.option(
    "--log-level",
    default="INFO",
    type=click.Choice(logging.LOG_LEVELS, case_sensitive=False),
)
def main(server, max_tasks, log_level):
    logging.set_log_level(log_level)
    asyncio.run(run(server, max_tasks))


if __name__ == "__main__":
    main()
